#import "BWMinuteTimer.h"


// --------------------------------------------------
// random utilities


// get the number where the Big Hand is pointing

static int getCurrentMinute ()
{
    time_t clock;
    struct tm *tm;
    char ook[1024];
    int minute;

    clock = time(NULL);
    tm = localtime(&clock);
    strftime (ook, 1024, "%M", tm); // get the minutes of the hour
    minute = atoi (ook);

    return (minute);

} // getCurrentMinute




// --------------------------------------------------
// class implementation


@implementation BWMinuteTimer


// our secret timer calls this.  We see if we've changed minute
// boundaries.  If so, call the user's method

- (void) timerTrampoline: (NSTimer *) ignored
{
    int minute;

    minute = getCurrentMinute ();

    if (minute != lastFire) {
	objc_msgSend (target, selector, self);
	lastFire = minute;
    }

} // timerTrampoline




// initialize a BWMinuteTimer object

- (BWMinuteTimer *) initWithTarget: (id) t
			  selector: (SEL) sel
			  userInfo: (id) uInfo
{
    if (self = [super init]) {
	target = t;
	selector = sel;

	// run our heartbeat timer every half second, and in that check
	// the curent minute.  If it changes, call the user method.
	// with a half-second run time, we should never be more than
	// half a second off of the top of the minute, +/- any
	// NSTimer precession.

	timer = [[NSTimer scheduledTimerWithTimeInterval: (NSTimeInterval) 0.5
			  target: self
			  selector: @selector(timerTrampoline:)
			  userInfo: uInfo
			  repeats: YES]
		    retain];

	// fire for the first time at the top of the next minute
	lastFire = getCurrentMinute ();
    }

    return (self);

} // initWithTimer




// public API for making a new BWMinuteTimer

+ (BWMinuteTimer *) scheduleTimerWithTarget: (id) t
				   selector: (SEL) sel
				   userInfo: (id) uInfo
{
    // WTF?  If I call this variable "timer", I get scads of warnings
    // that the variable hides instance variable.  Last I looked,
    // "+" means it's a *class* method, and so doesn't have access
    // to the instance variables
    // likewise with the argument names

    BWMinuteTimer *kimmer;

    kimmer = [self alloc];

    [kimmer initWithTarget: t
	    selector: sel
	    userInfo: uInfo];

    
    return (kimmer);
    
} // scheduledTimerWithTarget




// return the userInfo the creator of the timer wants us to keep around

- (id) userInfo
{
    return ([timer userInfo]);
} // userInfo




// free any resources we might have used

- (void) dealloc
{
    [timer release];

} // dealloc


@end // BWMinuteTimer

